==> powerprefs_0.4.9.orig.tar.gz: powerprefs-0.4.9/src/taghandler.c <==
/* ----------------------------------------------------------------------------
 * taghandler.c
 * funtions to get tagdata from and set tagdata to widgets
 *
 * Copyright 2002 Matthias Grimm
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include <pbb.h>

#include "init.h"
#include "taghandler.h"
#include "support.h"

extern struct basedata masterdata;

long
th_showtext (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET)
		gtk_label_set_text(GTK_LABEL(widget), (gchar*) data);
	return data;
}

long
th_showvalue (int mode, long tag, GtkWidget *widget, long data)
{
	char buffer[10];

	if (mode == THMODE_SET) {
		if (data == -1)
			gtk_label_set_text(GTK_LABEL(widget), _("not available"));
		else {
			snprintf(buffer, sizeof(buffer)-1, "%ld", data);
			gtk_label_set_text(GTK_LABEL(widget), (gchar*) buffer);
		}
	}
	return data;
}

long
th_textentry (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		gtk_entry_set_text(GTK_ENTRY(widget), (gchar*) data);
	} else
		data = (long) gtk_entry_get_text(GTK_ENTRY(widget));
	return data;
}

long
th_checkbox (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (widget), data ? TRUE : FALSE);
	} else
		data = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
	return data ? 1 : 0;
}

long
th_radiobutton (int mode, long tag, GtkWidget *widget, long data)
{
	GSList *radiolist;
	int n;

	if (mode == THMODE_SET) {
		radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
		if (g_slist_length(radiolist) > data) {
			for(n=g_slist_length(radiolist)-1; n > data; n--)
				radiolist = g_slist_next(radiolist);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (radiolist->data), TRUE);
		}
	} else {
		radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
		for(n=g_slist_length(radiolist)-1; n > 0; n--) {
			if ((gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(radiolist->data))) == TRUE)
				return n;
			radiolist = g_slist_next(radiolist);
		}
	}
	return data;
}

long
th_keyentry (int mode, long tag, GtkWidget *widget, long data)
{
	int key, mod;
	char arg[25];

	if (mode == THMODE_SET) {
		if (data != -1) {
			strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
			get_key_and_modifier(arg, &key, &mod);
			gtk_entry_set_text(GTK_ENTRY(widget), code_key_and_modifier(data, mod));
		}
	} else {
		strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
		get_key_and_modifier(arg, &key, &mod);
		data = key;
	}
	return data;
}

long
th_modentry (int mode, long tag, GtkWidget *widget, long data)
{
	int key, mod;
	char arg[25];

	if (mode == THMODE_SET) {
		if (data != -1) {
			strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
			get_key_and_modifier(arg, &key, &mod);
			gtk_entry_set_text(GTK_ENTRY(widget), code_key_and_modifier(key, data));
		}
	} else {
		strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
		get_key_and_modifier(arg, &key, &mod);
		data = mod;
	}
	return data;
}

long
th_spinbutton (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == -1) data = 0;
		gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget), (gdouble) data);
	} else
		data = (long) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(widget));
	return data;
}

long
th_combobox (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == -1) data = 0;
		gtk_combo_box_set_active (GTK_COMBO_BOX(widget), (gint) data);
	} else
		data = (long) gtk_combo_box_get_active (GTK_COMBO_BOX(widget));
	return data;
}

long
th_soundsystem (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == 3)
			gtk_label_set_text(GTK_LABEL(widget), "Advanced Linux Sound Architecture");
		else if (data == 2)
			gtk_label_set_text(GTK_LABEL(widget), "Open Sound System");
		else
			gtk_label_set_text(GTK_LABEL(widget), _("not available"));
}
	return data;
}


long
th_system (int mode, long tag, GtkWidget *widget, long data)
{
	struct basedata *base = (struct basedata*) &masterdata;
	GSList *radiolist;
	GtkWidget *wid;
	unsigned long val;

	switch(tag) {
		case TAG_SYSINFO:
			if (data == -1) data = 0;
			if (mode == THMODE_SET) {				
				/* disable sleep functions if sleep is not available */
				if ((data & SYSINFO_SLEEPSUPPORTED) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "pv_radio_ema")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
				
				/* hide pmud text if server is compiled without pmud support */
				if ((data & SYSINFO_PMUDSUPPORT) == 0)
					if ((wid = lookup_widget(base->mainwindow, "ab_itext_pmud")))
						gtk_widget_hide (wid);
				
				/* disable keyboard illumination and autoadjustment notetabs
				 * if a LMU is not available
				 */
				if ((data & SYSINFO_HAVELMU) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "vi_func_keyboardtab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_keyboardlab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_autoadjtab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_autoadjlab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
				
				/* disable button if server does not allow writing of configfile */
				if ((data & SYSINFO_CONFIGWRITABLE) == 0)
					if ((widget = lookup_widget(base->mainwindow, "main_btn_save")))
						gtk_widget_set_sensitive (widget, FALSE);

				/* disable OSS notetab if a OSS mixer module is not available
				 */
				if ((data & SYSINFO_HAVEOSS) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "au_func_osstab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "au_func_osslab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}

				/* disable ALSA notetab if a ALSA mixer module is not available
				 */
				if ((data & SYSINFO_HAVEALSA) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "au_func_alsatab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "au_func_alsalab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
	
				/* disable sound system options if only one soundsystem is available */
				val = SYSINFO_HAVEOSS | SYSINFO_HAVEALSA;
				if ((data & val) != val)
					if ((widget = lookup_widget(base->mainwindow, "au_radio_sound"))) {
						radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
						while (radiolist) {
							gtk_widget_set_sensitive (GTK_WIDGET (radiolist->data), FALSE);
							radiolist = g_slist_next(radiolist);
						}
					}
			}
			break;
	}
	return data;
}

long
th_profile (int mode, long tag, GtkWidget *widget, long data)
{
	struct basedata *base = (struct basedata*) &masterdata;

	if (data == -1)   /* tag is not supported */
		return data;
	
	switch(tag) {
		case TAG_ONAC_POLICY:
			if (mode)	base->ac.policy = data;
			else		data = base->ac.policy;
			break;
		case TAG_ONAC_TIMERACTION:
			if (mode)	base->ac.timeraction = data;
			else		data = base->ac.timeraction;
			break;
		case TAG_ONAC_COVERACTION:
			if (mode)	base->ac.coveraction = data;
			else		data = base->ac.coveraction;
			break;
		case TAG_ONAC_KEYACTION:
			if (mode)	base->ac.keyaction = data;
			else		data = base->ac.keyaction;
			break;
		case TAG_ONAC_TIMESUSPEND:
			if (mode)	base->ac.timesuspend = (long) data / 10;
			else		data = base->ac.timesuspend * 10;
			break;
		case TAG_ONAC_TIMEDIM:
			if (mode)	base->ac.timedim = (long) data / 10;
			else		data = base->ac.timedim * 10;
			break;
		case TAG_ONBATT_POLICY:
			if (mode)	base->battery.policy = data;
			else		data = base->battery.policy;
			break;
		case TAG_ONBATT_TIMERACTION:
			if (mode)	base->battery.timeraction = data;
			else		data = base->battery.timeraction;
			break;
		case TAG_ONBATT_COVERACTION:
			if (mode)	base->battery.coveraction = data;
			else		data = base->battery.coveraction;
			break;
		case TAG_ONBATT_KEYACTION:
			if (mode)	base->battery.keyaction = data;
			else		data = base->battery.keyaction;
			break;
		case TAG_ONBATT_TIMESUSPEND:
			if (mode)	base->battery.timesuspend = (long) data / 10;
			else		data = base->battery.timesuspend * 10;
			break;
		case TAG_ONBATT_TIMEDIM:
			if (mode)	base->battery.timedim = (long) data / 10;
			else		data = base->battery.timedim * 10;
			break;
	}

	if (mode == THMODE_SET)
		update_profile_gui();
	return data;
}

long
th_dummy (int mode, long tag, GtkWidget *widget, long data)
{
	return data;
}

void
update_profile_gui()
{
	struct basedata *base = (struct basedata*) &masterdata;
	GtkWidget *widget;

	if ((widget = lookup_widget(base->mainwindow, "pv_combo_policy")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->policy));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_timeraction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->timeraction));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_coveraction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->coveraction));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_keyaction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->keyaction));
	if ((widget = lookup_widget(base->mainwindow, "pv_spin_sleeptime")))
		(void) th_spinbutton (THMODE_SET, 0, widget, base->profile->timesuspend);
	if ((widget = lookup_widget(base->mainwindow, "pv_spin_dimtime")))
		(void) th_spinbutton (THMODE_SET, 0, widget, base->profile->timedim);
}

void
get_key_and_modifier(char *arg, int *key, int *mod)
{
	char *tag;
	int k;

	*mod = 0; *key = 0;
	tag = strtok(arg, "+\n");
	while (tag) {
		k = atoi (tag) & 255;
		if (k == 0) {
			while (*tag == ' ') tag++;   /* ignore leading spaces */
			if (!strncmp("shift", tag, 5))
				*mod |= MOD_SHIFT;
			else if (!strncmp("alt", tag, 3))
				*mod |= MOD_ALT;
			else if (!strncmp("ctrl", tag, 4))
				*mod |= MOD_CTRL;
		} else
			*key = k;
		tag = strtok(0, "+\n");
	}
}

char*
code_key_and_modifier (int key, int mod)
{
	static char buffer[30];

	snprintf (buffer, sizeof(buffer)-1, "%d", (key & 255));
	if (mod & MOD_SHIFT)
		strcat (buffer, " + shift");
	if (mod & MOD_ALT)
		strcat (buffer, " + alt");
	if (mod & MOD_CTRL)
		strcat (buffer, " + ctrl");
	return buffer;
}

==> powerprefs_0.5.1.orig.tar.gz: powerprefs-0.5.1/src/taghandler.c <==
/* ----------------------------------------------------------------------------
 * taghandler.c
 * funtions to get tagdata from and set tagdata to widgets
 *
 * Copyright 2002 Matthias Grimm
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include <pbb.h>

#include "init.h"
#include "taghandler.h"
#include "support.h"

extern struct basedata masterdata;

long
th_showtext (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET)
		gtk_label_set_text(GTK_LABEL(widget), (gchar*) data);
	return data;
}

long
th_showvalue (int mode, long tag, GtkWidget *widget, long data)
{
	char buffer[10];

	if (mode == THMODE_SET) {
		if (data == -1)
			gtk_label_set_text(GTK_LABEL(widget), _("not available"));
		else {
			snprintf(buffer, sizeof(buffer)-1, "%ld", data);
			gtk_label_set_text(GTK_LABEL(widget), (gchar*) buffer);
		}
	}
	return data;
}

long
th_showpvalue (int mode, long tag, GtkWidget *widget, long data)
{
	char buffer[10];

	if (mode == THMODE_SET) {
		if (data == -1)
			gtk_label_set_text(GTK_LABEL(widget), _("not available"));
		else {
			snprintf(buffer, sizeof(buffer)-1, "%ld %%", data);
			gtk_label_set_text(GTK_LABEL(widget), (gchar*) buffer);
		}
	}
	return data;
}


long
th_textentry (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		gtk_entry_set_text(GTK_ENTRY(widget), (gchar*) data);
	} else
		data = (long) gtk_entry_get_text(GTK_ENTRY(widget));
	return data;
}

long
th_checkbox (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (widget), data ? TRUE : FALSE);
	} else
		data = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
	return data ? 1 : 0;
}

long
th_radiobutton (int mode, long tag, GtkWidget *widget, long data)
{
	GSList *radiolist;
	int n;

	if (mode == THMODE_SET) {
		radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
		if (g_slist_length(radiolist) > data) {
			for(n=g_slist_length(radiolist)-1; n > data; n--)
				radiolist = g_slist_next(radiolist);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (radiolist->data), TRUE);
		}
	} else {
		radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
		for(n=g_slist_length(radiolist)-1; n > 0; n--) {
			if ((gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(radiolist->data))) == TRUE)
				return n;
			radiolist = g_slist_next(radiolist);
		}
	}
	return data;
}

long
th_keyentry (int mode, long tag, GtkWidget *widget, long data)
{
	int key, mod;
	char arg[25];

	if (mode == THMODE_SET) {
		if (data != -1) {
			strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
			get_key_and_modifier(arg, &key, &mod);
			gtk_entry_set_text(GTK_ENTRY(widget), code_key_and_modifier(data, mod));
		}
	} else {
		strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
		get_key_and_modifier(arg, &key, &mod);
		data = key;
	}
	return data;
}

long
th_modentry (int mode, long tag, GtkWidget *widget, long data)
{
	int key, mod;
	char arg[25];

	if (mode == THMODE_SET) {
		if (data != -1) {
			strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
			get_key_and_modifier(arg, &key, &mod);
			gtk_entry_set_text(GTK_ENTRY(widget), code_key_and_modifier(key, data));
		}
	} else {
		strncpy (arg, (char*) gtk_entry_get_text(GTK_ENTRY(widget)), sizeof(arg));
		get_key_and_modifier(arg, &key, &mod);
		data = mod;
	}
	return data;
}

long
th_spinbutton (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == -1) data = 0;
		gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget), (gdouble) data);
	} else
		data = (long) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(widget));
	return data;
}

long
th_combobox (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == -1) data = 0;
		gtk_combo_box_set_active (GTK_COMBO_BOX(widget), (gint) data);
	} else
		data = (long) gtk_combo_box_get_active (GTK_COMBO_BOX(widget));
	return data;
}

long
th_soundsystem (int mode, long tag, GtkWidget *widget, long data)
{
	if (mode == THMODE_SET) {
		if (data == 3)
			gtk_label_set_text(GTK_LABEL(widget), "Advanced Linux Sound Architecture");
		else if (data == 2)
			gtk_label_set_text(GTK_LABEL(widget), "Open Sound System");
		else
			gtk_label_set_text(GTK_LABEL(widget), _("not available"));
}
	return data;
}


long
th_system (int mode, long tag, GtkWidget *widget, long data)
{
	struct basedata *base = (struct basedata*) &masterdata;
	GSList *radiolist;
	GtkWidget *wid;
	unsigned long val;

	switch(tag) {
		case TAG_SYSINFO:
			if (data == -1) data = 0;
			if (mode == THMODE_SET) {				
				/* disable sleep functions if sleep is not available */
				if ((data & SYSINFO_SLEEPSUPPORTED) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "pv_radio_ema")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
				
				/* hide pmud text if server is compiled without pmud support */
				if ((data & SYSINFO_PMUDSUPPORT) == 0)
					if ((wid = lookup_widget(base->mainwindow, "ab_itext_pmud")))
						gtk_widget_hide (wid);
				
				/* disable keyboard illumination and autoadjustment notetabs
				 * if a LMU is not available
				 */
				if ((data & SYSINFO_HAVELMU) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "vi_func_keyboardtab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_keyboardlab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_autoadjtab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "vi_func_autoadjlab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
				
				/* disable button if server does not allow writing of configfile */
				if ((data & SYSINFO_CONFIGWRITABLE) == 0)
					if ((widget = lookup_widget(base->mainwindow, "main_btn_save")))
						gtk_widget_set_sensitive (widget, FALSE);

				/* disable OSS notetab if a OSS mixer module is not available
				 */
				if ((data & SYSINFO_HAVEOSS) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "au_func_osstab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "au_func_osslab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}

				/* disable ALSA notetab if a ALSA mixer module is not available
				 */
				if ((data & SYSINFO_HAVEALSA) == 0) {
					if ((wid = lookup_widget(base->mainwindow, "au_func_alsatab")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "au_func_alsalab")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
	
				/* disable sound system options if only one soundsystem is available */
				val = SYSINFO_HAVEOSS | SYSINFO_HAVEALSA;
				if ((data & val) != val)
					if ((widget = lookup_widget(base->mainwindow, "au_radio_sound"))) {
						radiolist = gtk_radio_button_get_group (GTK_RADIO_BUTTON(widget));
						while (radiolist) {
							gtk_widget_set_sensitive (GTK_WIDGET (radiolist->data), FALSE);
							radiolist = g_slist_next(radiolist);
						}
					}

				/* disable extra keyboard mode if we have an ADB keyboard */
				if ((data & SYSINFO_PB_USB_KBD) == 0)
					if ((wid = lookup_widget(base->mainwindow, "ms_radio_keyboard")))
						gtk_widget_set_sensitive (wid, FALSE);

				/* disable trackpad configuration if we have an USB trackpad */
				if ((data & SYSINFO_PB_USB_TPAD) == SYSINFO_PB_USB_TPAD) {
					if ((wid = lookup_widget(base->mainwindow, "ms_trackpad_mode")))
						gtk_widget_set_sensitive (wid, FALSE);
					if ((wid = lookup_widget(base->mainwindow, "ms_trackpad_keys")))
						gtk_widget_set_sensitive (wid, FALSE);
				}
			}
			break;
	}
	return data;
}

long
th_profile (int mode, long tag, GtkWidget *widget, long data)
{
	struct basedata *base = (struct basedata*) &masterdata;

	if (data == -1)   /* tag is not supported */
		return data;
	
	switch(tag) {
		case TAG_ONAC_POLICY:
			if (mode)	base->ac.policy = data;
			else		data = base->ac.policy;
			break;
		case TAG_ONAC_TIMERACTION:
			if (mode)	base->ac.timeraction = data;
			else		data = base->ac.timeraction;
			break;
		case TAG_ONAC_COVERACTION:
			if (mode)	base->ac.coveraction = data;
			else		data = base->ac.coveraction;
			break;
		case TAG_ONAC_KEYACTION:
			if (mode)	base->ac.keyaction = data;
			else		data = base->ac.keyaction;
			break;
		case TAG_ONAC_TIMESUSPEND:
			if (mode)	base->ac.timesuspend = (long) data / 10;
			else		data = base->ac.timesuspend * 10;
			break;
		case TAG_ONAC_TIMEDIM:
			if (mode)	base->ac.timedim = (long) data / 10;
			else		data = base->ac.timedim * 10;
			break;
		case TAG_ONBATT_POLICY:
			if (mode)	base->battery.policy = data;
			else		data = base->battery.policy;
			break;
		case TAG_ONBATT_TIMERACTION:
			if (mode)	base->battery.timeraction = data;
			else		data = base->battery.timeraction;
			break;
		case TAG_ONBATT_COVERACTION:
			if (mode)	base->battery.coveraction = data;
			else		data = base->battery.coveraction;
			break;
		case TAG_ONBATT_KEYACTION:
			if (mode)	base->battery.keyaction = data;
			else		data = base->battery.keyaction;
			break;
		case TAG_ONBATT_TIMESUSPEND:
			if (mode)	base->battery.timesuspend = (long) data / 10;
			else		data = base->battery.timesuspend * 10;
			break;
		case TAG_ONBATT_TIMEDIM:
			if (mode)	base->battery.timedim = (long) data / 10;
			else		data = base->battery.timedim * 10;
			break;
	}

	if (mode == THMODE_SET)
		update_profile_gui();
	return data;
}

long
th_dummy (int mode, long tag, GtkWidget *widget, long data)
{
	return data;
}

void
update_profile_gui()
{
	struct basedata *base = (struct basedata*) &masterdata;
	GtkWidget *widget;

	if ((widget = lookup_widget(base->mainwindow, "pv_combo_policy")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->policy));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_timeraction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->timeraction));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_coveraction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->coveraction));
	if ((widget = lookup_widget(base->mainwindow, "pv_combo_keyaction")))
		(void) th_combobox (THMODE_SET, 0, widget, (base->profile->keyaction));
	if ((widget = lookup_widget(base->mainwindow, "pv_spin_sleeptime")))
		(void) th_spinbutton (THMODE_SET, 0, widget, base->profile->timesuspend);
	if ((widget = lookup_widget(base->mainwindow, "pv_spin_dimtime")))
		(void) th_spinbutton (THMODE_SET, 0, widget, base->profile->timedim);
}

void
get_key_and_modifier(char *arg, int *key, int *mod)
{
	char *tag;
	int k;

	*mod = 0; *key = 0;
	tag = strtok(arg, "+\n");
	while (tag) {
		k = atoi (tag) & 255;
		if (k == 0) {
			while (*tag == ' ') tag++;   /* ignore leading spaces */
			if (!strncmp("shift", tag, 5))
				*mod |= MOD_SHIFT;
			else if (!strncmp("alt", tag, 3))
				*mod |= MOD_ALT;
			else if (!strncmp("ctrl", tag, 4))
				*mod |= MOD_CTRL;
		} else
			*key = k;
		tag = strtok(0, "+\n");
	}
}

char*
code_key_and_modifier (int key, int mod)
{
	static char buffer[30];

	snprintf (buffer, sizeof(buffer)-1, "%d", (key & 255));
	if (mod & MOD_SHIFT)
		strcat (buffer, " + shift");
	if (mod & MOD_ALT)
		strcat (buffer, " + alt");
	if (mod & MOD_CTRL)
		strcat (buffer, " + ctrl");
	return buffer;
}
